import { Module, Provider } from '@nestjs/common';
import { CqrsModule } from '@nestjs/cqrs';
import {
  DIRECTION_ENCODER,
  GEODESIC,
  GEOGRAPHY_CONFIGURATION_REPOSITORY,
  GEOROUTER,
} from './geography.di-tokens';
import { PostgresDirectionEncoder } from './infrastructure/postgres-direction-encoder';
import { GetBasicRouteController } from './interface/controllers/get-basic-route.controller';
import { RouteMapper } from './route.mapper';
import { Geodesic } from './infrastructure/geodesic';
import { GraphhopperGeorouter } from './infrastructure/graphhopper-georouter';
import { HttpModule } from '@nestjs/axios';
import { GetRouteQueryHandler } from './core/application/queries/get-route/get-route.query-handler';
import { GetDetailedRouteController } from './interface/controllers/get-detailed-route.controller';
import { ConfigurationRepository } from '@mobicoop/configuration-module';

const queryHandlers: Provider[] = [GetRouteQueryHandler];

const mappers: Provider[] = [RouteMapper];

const adapters: Provider[] = [
  {
    provide: GEOGRAPHY_CONFIGURATION_REPOSITORY,
    useClass: ConfigurationRepository,
  },
  {
    provide: DIRECTION_ENCODER,
    useClass: PostgresDirectionEncoder,
  },
  {
    provide: GEOROUTER,
    useClass: GraphhopperGeorouter,
  },
  {
    provide: GEODESIC,
    useClass: Geodesic,
  },
  GetBasicRouteController,
  GetDetailedRouteController,
];

@Module({
  imports: [CqrsModule, HttpModule],
  providers: [...queryHandlers, ...mappers, ...adapters],
  exports: [
    RouteMapper,
    DIRECTION_ENCODER,
    GetBasicRouteController,
    GetDetailedRouteController,
  ],
})
export class GeographyModule {}
